import React from 'react'
import { fromJS } from 'immutable'
import { useDrag, useDrop } from 'react-dnd'
import { Popover } from 'antd'
import { CloseOutlined, EditOutlined } from '@ant-design/icons'

import getForm from './options'
import asyncIconComponent from '@/utils/asyncIconComponent'
import './index.scss'

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const PasteForms = asyncIconComponent(() => import('@/menu/components/share/pasteforms'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))

const Card = ({ id, card, sort, labelSize, active, moveCard, findCard, closeCard, selectCard, updateGroup, pasteForm }) => {
  const originalIndex = findCard(id).index
  const [{ isDragging }, drag] = useDrag({
    item: { type: 'form', id, originalIndex },
    collect: monitor => ({
      isDragging: monitor.isDragging(),
    }),
  })
  const [, drop] = useDrop({
    accept: 'form',
    canDrop: () => true,
    drop: (item) => {
      const { id: draggedId, originalIndex } = item

      if (originalIndex === undefined) {
        item.dropTargetId = id
      } else if (draggedId && draggedId !== id) {
        const { index: overIndex } = findCard(id)
        moveCard(draggedId, overIndex)
      }
    }
  })
  const opacity = isDragging ? 0.5 : 1

  const close = () => {
    closeCard(id)
  }

  const select = () => {
    selectCard(id)
  }

  const getForms = () => {
    return getForm(card)
  }

  const updateSetting = (res) => {
    let _card = fromJS(card).toJS()
    
    let buttons = res.buttons

    delete res.buttons

    if (buttons) {
      res.prevEnable = buttons.includes('prevEnable') ? 'true' : 'false'
      res.subEnable = buttons.includes('subEnable') ? 'true' : 'false'
      res.nextEnable = buttons.includes('nextEnable') ? 'true' : 'false'
    }

    if (res.prevEnable) {
      _card.prevButton.enable = res.prevEnable
      delete res.prevEnable
    }
    if (res.subEnable) {
      _card.subButton.enable = res.subEnable
      delete res.subEnable
    }
    if (res.nextEnable) {
      _card.nextButton.enable = res.nextEnable
      delete res.nextEnable
    }

    _card.setting = res

    updateGroup(_card)
  }

  let style = {fontSize: labelSize}
  let s = labelSize * 1.5 + 'px'
  let sortStyle = {width: s, height: s, lineHeight: s, borderRadius: s}

  return (
    <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
      <div className="mk-popover-control">
        <NormalForm title="分组编辑" width={850} update={updateSetting} getForms={getForms}>
          <EditOutlined style={{color: '#1890ff'}} title="编辑"/>
        </NormalForm>
        <CopyComponent type="formgroup" card={card}/>
        <PasteForms config={card} updateConfig={(res) => pasteForm(res, id)} />
        <CloseOutlined className="close" type="close" onClick={close} />
      </div>
    } trigger="hover">
      <div className={'page-card ' + (active ? 'active' : '')} onClick={select} style={{ opacity: opacity}}>
        <div ref={node => drag(drop(node))} style={style}>
          <span className="form-sort" style={sortStyle}>{sort}</span>
          {card.setting.title}
        </div>
      </div>
    </Popover>
  )
}
export default Card
